{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Volume Transfer Details" %}{% endblock %}

{% block main %}
    {% include 'project/volumes/_show_transfer.html' %}
{% endblock %}
