{% extends 'base.html' %}
{% load i18n %}

{% block title %}
  {{ page_title }}
{% endblock %}

{% block page_header %}
  {% include 'horizon/common/_detail_header.html' %}
{% endblock %}

{% block main %}
  <div class="row">
    <div class="col-sm-12">
      {{ tab_group.render }}
    </div>
  </div>
{% endblock %}
