{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <h3>{% trans "Description:" %}</h3>
  <p>{% trans "Create a identity provider protocol." %}</p>
  <p>{% blocktrans %} A protocol entry contains information that dictates which mapping rules to use for a given incoming request. An Identity Provider may have multiple supported protocols.{% endblocktrans %}</p>
{% endblock %}