{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
    <h3>{% trans "Description:" %}</h3>
    <p>{% blocktrans trimmed %}
      Ownership of a volume can be transferred from one project to another.
      Accepting a transfer requires obtaining the Transfer ID and Authorization
      Key from the donor. This is equivalent to the <tt>openstack volume transfer
      request accept</tt> command.
      {% endblocktrans %}</p>
{% endblock %}
