/**
 * Copyright 2016 Rackspace
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain
 * a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

(function() {
  'use strict';

  angular
    .module('horizon.dashboard.identity.roles')
    .factory('horizon.dashboard.identity.roles.role-schema', schema);

  schema.$inject = [
    'horizon.framework.util.i18n.gettext'
  ];

  function schema(gettext) {
    return {
      type: 'object',
      properties: {
        name: {
          title: gettext('Name'),
          type: 'string'
        }
      },
      required: ['name']
    };
  }
})();
